@@ -9,17 +9,17 @@ from ..grpc_gen import status_pb2
 class Prepare:
 
     @classmethod
-    def table_name(cls, table_name):
+    def collection_name(cls, collection_name):
 
-        return grpc_types.TableName(table_name=table_name)
+        return grpc_types.CollectionName(collection_name=collection_name)
 
     @classmethod
-    def table_schema(cls, table_name, dimension, index_file_size, metric_type, param):
+    def collection_schema(cls, collection_name, dimension, index_file_size, metric_type, param):
         """
         :type param: dict
         :param param: (Required)
 
-            `example param={'table_name': 'name',
+            `example param={'collection_name': 'name',
                             'dimension': 16,
                             'index_file_size': 1024
                             'metric_type': MetricType.L2
@@ -28,11 +28,11 @@ class Prepare:
         :return: ttypes.TableSchema object
         """
 
-        _param = grpc_types.TableSchema(status=status_pb2.Status(error_code=0, reason='Client'),
-                                        table_name=table_name,
-                                        dimension=dimension,
-                                        index_file_size=index_file_size,
-                                        metric_type=metric_type)
+        _param = grpc_types.CollectionSchema(status=status_pb2.Status(error_code=0, reason='Client'),
+                                             collection_name=collection_name,
+                                             dimension=dimension,
+                                             index_file_size=index_file_size,
+                                             metric_type=metric_type)
 
         if param:
             param_str = ujson.dumps(param)
@@ -41,13 +41,12 @@ class Prepare:
         return _param
 
     @classmethod
-    def insert_param(cls, table_name, vectors, partition_tag, ids=None, params=None, **kwargs):
-
+    def insert_param(cls, collection_name, vectors, partition_tag, ids=None, params=None, **kwargs):
         if ids is None:
-            _param = grpc_types.InsertParam(table_name=table_name, partition_tag=partition_tag)
+            _param = grpc_types.InsertParam(collection_name=collection_name, partition_tag=partition_tag)
         else:
             _param = grpc_types.InsertParam(
-                table_name=table_name,
+                collection_name=collection_name,
                 row_id_array=ids,
                 partition_tag=partition_tag)
 
@@ -64,10 +63,10 @@ class Prepare:
         return _param
 
     @classmethod
-    def index_param(cls, table_name, index_type, params):
+    def index_param(cls, collection_name, index_type, params):
 
         _param = grpc_types.IndexParam(status=status_pb2.Status(error_code=0, reason='Client'),
-                                       table_name=table_name,
+                                       collection_name=collection_name,
                                        index_type=index_type)
         params = params or dict()
         params_str = ujson.dumps(params)
@@ -76,10 +75,10 @@ class Prepare:
         return _param
 
     @classmethod
-    def search_param(cls, table_name, topk, query_records, partitions, params):
+    def search_param(cls, collection_name, topk, query_records, partitions, params):
 
         search_param = grpc_types.SearchParam(
-            table_name=table_name,
+            collection_name=collection_name,
             topk=topk,
             partition_tag_array=partitions
         )
@@ -97,11 +96,12 @@ class Prepare:
         return search_param
 
     @classmethod
-    def search_by_id_param(cls, table_name, top_k, id_, partition_tag_array, params):
+    def search_by_ids_param(cls, collection_name, ids, top_k, partition_tag_array, params):
         _param = grpc_types.SearchByIDParam(
-            table_name=table_name, id=id_, topk=top_k,
-            partition_tag_array=partition_tag_array
+            collection_name=collection_name, id_array=ids, topk=top_k,
         )
+        if partition_tag_array:
+            _param.partition_tag_array[:] = partition_tag_array
 
         params = params or dict()
         params_str = ujson.dumps(params)
@@ -110,9 +110,9 @@ class Prepare:
         return _param
 
     @classmethod
-    def search_vector_in_files_param(cls, table_name, query_records, topk, ids, params):
-        _search_param = Prepare.search_param(table_name, topk, query_records,
-                                             partitions=[], params=params)
+    def search_vector_in_files_param(cls, collection_name, query_records, topk, ids, params):
+        _search_param = Prepare.search_param(collection_name, topk, query_records,
+                                             partitions=None, params=params)
 
         return grpc_types.SearchInFilesParam(
             file_id_array=ids,
@@ -125,20 +125,20 @@ class Prepare:
         return grpc_types.Command(cmd=cmd)
 
     @classmethod
-    def partition_param(cls, table_name, tag):
+    def partition_param(cls, collection_name, tag):
 
-        return grpc_types.PartitionParam(table_name=table_name, tag=tag)
+        return grpc_types.PartitionParam(collection_name=collection_name, tag=tag)
 
     @classmethod
-    def delete_by_id_param(cls, table_name, id_array):
+    def delete_by_id_param(cls, collection_name, id_array):
 
-        return grpc_types.DeleteByIDParam(table_name=table_name, id_array=id_array)
+        return grpc_types.DeleteByIDParam(collection_name=collection_name, id_array=id_array)
 
     @classmethod
-    def flush_param(cls, table_names):
+    def flush_param(cls, collection_names):
 
-        return grpc_types.FlushParam(table_name_array=table_names)
+        return grpc_types.FlushParam(collection_name_array=collection_names)
 
     @classmethod
-    def compact_param(cls, table_name):
-        return grpc_types.TableName(table_name=table_name)
+    def compact_param(cls, collection_name):
+        return grpc_types.CollectionName(collection_name=collection_name)
